#ifndef Sequence_H
#define Sequence_H
//------------------------------------------------------------------------------
/* File: Sequence.h
  
   Desc: This is the header file for the Sequence. It assists in writing the
         sequence header informamtion and holds a BitWriter Object that the
         Controller can use.
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>


class CMPEGSeqHeader
{

public:
    ulong32 StartCode;           //32
    uint16  HorizontalSize;      //12  
    uint16  VerticalSize;        //12
    ubyte8  AspectRatio;         //4
    ubyte8  FrameRateCode;       //4
    uint16  BitRate;             //18 
    ubyte8  MarkerBit;           //1, = 1
    ubyte8  VBVBufferSize;       //10
    ubyte8  ConstraindParam;     //1 
    ubyte8  LoadIntraQntMat;     //1
    ubyte8  LoadInterQntMat;     //1

    CMPEGSeqHeader()
    {
        StartCode       = 0x000001B3;
        HorizontalSize  = 320;
        VerticalSize    = 240;
        AspectRatio     = 2;   // Say we are 4:3
        FrameRateCode   = 4;   // Say 30 Frames/Sec.
        BitRate         = (uint16)(8000000.0/400.0); // For Now
        MarkerBit       = 1;
        VBVBufferSize   = 112; // This could be interesting, First 10 bits
        ConstraindParam = 0;   // No constraints
        LoadIntraQntMat = 0;   // Use Default Values
        LoadInterQntMat = 0;   // Use Default Values
    }

    void SetHorzSize( uint16 iHSize ){ HorizontalSize = iHSize; }
    void SetVertSize( uint16 iVSize ){ VerticalSize   = iVSize; }
    void SetBitRate ( double dBitRate ){ BitRate = (uint16)(dBitRate/400.0);  }
    
    void Write( CGSBitWriter &);
};

class CMPEGExtSeqHeader
{
public:
    ulong32  ExtStartCode;        //32
    ubyte8   SequenceID;          //4
    ubyte8   ProfileAndLevel;     //8
    ubyte8   IsProgressive;       //1
    ubyte8   ChromaFormat;        //2
    ubyte8   ExtHorizSize;        //2
    ubyte8   ExtVertSize;         //2
    uint16   ExtBitRate;          //12
    ubyte8   MarkerBit;           //1
    ubyte8   ExtVBVBuffSize;      //8
    ubyte8   IsLowDelay;          //1
    ubyte8   NFrameRate;          //2
    ubyte8   DFrameRate;          //5

    CMPEGExtSeqHeader()
    {
        ExtStartCode    = 0x000001B5;  //All Sequence Extension Start Codes
        SequenceID      = 1;           //MPEG2 Extension
        ProfileAndLevel = 0x48;        //Main & Main
        IsProgressive   = 1;           //For Now
        ChromaFormat    = 2;           //YUY2, ie. YUV2 4:2:2
        ExtHorizSize    = 0;           //We are only 2048 
        ExtVertSize     = 0;           //We are only 2048  
        ExtBitRate      = 0;           // NEED TO CHECK THIS!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
        MarkerBit       = 1;
        ExtVBVBuffSize  = 0;           // For now, unless want to request large size, Next 8 bits
        IsLowDelay      = 1;           // We plan to use B pictures, so Zero, Sec. 6.3.5
        NFrameRate      = 0;           // No extension values used for FrameRate FR = FRV*(n+1)/(d+1)
        DFrameRate      = 0;           // No extension values used for FrameRate
    }

    void Write( CGSBitWriter &);
    void SetChromaFormat( ubyte8 ucChromaFormat ){ ChromaFormat = ucChromaFormat; }
};


class CMPEGExtDisplayHeader
{
public:
    ulong32 ExtStartCode;          //32
    ubyte8  DisplaySequence;       //4
    ubyte8  VideoFormat;           //3
    ubyte8  ColorDescription;      //1
    ubyte8  ColorPrimaries;        //8
    ubyte8  TransCharacteristics;  //8
    ubyte8  MatrixCoeffs;          //8
    uint16  HorizontalSize;        //14
    ubyte8  MarkerBit;             //1
    uint16  VerticalSize;          //14

    CMPEGExtDisplayHeader()
    {
        ExtStartCode          = 0x000001B5;      //All Sequence Extensions
        DisplaySequence       = 2;               // ID for this
        VideoFormat           = 2;               // NTSC
        ColorDescription      = 1;               // Indicate Presence of Color Primaries
        ColorPrimaries        = 5;               // ITU-R BT.470-2 System B,G, Table 6-7
        TransCharacteristics  = 5;               // Same as above, Table 6-8
        MatrixCoeffs          = 4;               // FCC, Table 6-9
        HorizontalSize        = 704;             // Interesting! From PAR File!
        MarkerBit             = 1;               //
        VerticalSize          = 480;             // Interesting!  From PAR File!
    }

    void SetHorzSize( uint16 iHSize ){ HorizontalSize = iHSize; }
    void SetVertSize( uint16 iVSize ){ VerticalSize   = iVSize; }
    void Write( CGSBitWriter &);
};



class CMPEGUserData
{
public:
    ulong32      UserDataCode;
    const sbyte8 *pUserData;

    CMPEGUserData()
    {
        UserDataCode = 0x000001B2;
        pUserData    = "GS MPEG2 Encoder Version 0.9";
    }

    void Write( CGSBitWriter &);
};

    
class CMPEGGOPHeader
{
public:
    ulong32 GOPStartCode;

    CMPEGGOPHeader(){ GOPStartCode = 0x000001B8; }

    void Write( CGSBitWriter &BitWriter, ulong32 uTimeCode, bool isClosed );
};


class CMPEGSeqEnd
{
public:
    ulong32 SequenceEndCode;

    CMPEGSeqEnd()
    {
        SequenceEndCode = 0x000001B7;
    }

    void Write( CGSBitWriter &);
};


class CGSSequence
{
    private:
        CMPEGSeqHeader        SequenceHeader;
        CMPEGExtSeqHeader     ExtSequenceHeader;
        CMPEGExtDisplayHeader DisplayHeader;
        CMPEGUserData         UserData;
        CMPEGGOPHeader        GOPHeader;
        CMPEGSeqEnd           SequenceEnd;

        ulong32               iPictureCount;
        bool                  isInitialized;

        CGSBitWriter          &BitWriter;
        CGSVLCoder            &VLCoder;
        CGSMotionCalculator   &MotionCalculator;

    public:
        CGSSequence( CGSBitWriter &rBitWriter, CGSVLCoder &rVLCoder, CGSMotionCalculator &rMCalctor ):
            BitWriter( rBitWriter ), VLCoder( rVLCoder ), MotionCalculator( rMCalctor )
        {
            isInitialized = false;
        }

        ~CGSSequence()
        {
        }

        ulong32 GetPictureCount(){ return iPictureCount; }
        void    IncrementPictureCount(){ iPictureCount++; }
        void    Initialize( sint16 imageWidth, sint16 imageHeight );
        void    NewGOP()
        {
            iPictureCount = 0;
            GOPHeader.Write( BitWriter, 0, false );
        }

        void    Finalize();
};


#endif //#ifndef Sequence_H
