//------------------------------------------------------------------------------
// File: GSEncoder.cpp
/*
   Desc: Implementation File for the GSEncoder

   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <streams.h>      // Active Movie (includes windows.h)
#include <initguid.h>     // declares DEFINE_GUID to declare an EXTERN_C const.

#include <GSEncoder.h>

#pragma warning(disable: 4800)


// Setup data - allows the self-registration to work.
const AMOVIESETUP_MEDIATYPE sudPinTypes =
{ &MEDIATYPE_NULL          // clsMajorType
, &MEDIASUBTYPE_NULL };  // clsMinorType


const AMOVIESETUP_PIN psudGSEncoderPins[] =
{
    {
        L"Input",             // strName
        FALSE,               // bRendered
        FALSE,               // bOutput
        FALSE,               // bZero
        FALSE,               // bMany
        &CLSID_NULL,         // clsConnectsToFilter
        L"",                 // strConnectsToPin
        1,                   // nTypes
        &sudPinTypes        // lpTypes
    },
    { 
        L"Output",           // strName
        FALSE,               // bRendered
        TRUE,                // bOutput
        FALSE,               // bZero
        FALSE,               // bMany
        &CLSID_NULL,         // clsConnectsToFilter
        L"",                 // strConnectsToPin
        1,                   // nTypes
        &sudPinTypes        // lpTypes
    }
};

const AMOVIESETUP_FILTER sudGSEncoder =
{ 
    &CLSID_GSEncoder,          // clsID
    L"GSMPEG2 Encoder",        // strName
    MERIT_DO_NOT_USE,          // dwMerit
    2,                         // nPins
    psudGSEncoderPins
};       // lpPin


// Needed for the CreateInstance mechanism
CFactoryTemplate g_Templates[]=
{
    {
        L"GSMPEG2 Encoder",
        &CLSID_GSEncoder,
        CGSEncoder::CreateInstance,
        NULL,
        &sudGSEncoder
    }
};

int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);


/******************************Public*Routine******************************\
*
* Exported entry points for registration and unregistration (in this case 
* they only call through to default implmentations).
*
*\**************************************************************************/
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2(TRUE);
} 

STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2(FALSE);
}

//
// CreateInstance
//
// Provide the way for COM to create a CGSEncoder object
//
CUnknown * WINAPI CGSEncoder::CreateInstance(LPUNKNOWN punk, HRESULT *phr) 
{
  // assuming we don't want to modify the data
    CGSEncoder *pNewObject = new CGSEncoder(punk, phr, FALSE);

    if(pNewObject == NULL)
    {
        *phr = E_OUTOFMEMORY;
    }

    return pNewObject;   
} // CreateInstance


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------

CGSEncoder::CGSEncoder( IUnknown * pOuter, HRESULT * phr, BOOL ModifiesData )
: CTransInPlaceFilter( TEXT("GSEncoder"), (IUnknown*) pOuter, CLSID_GSEncoder, phr, (BOOL)ModifiesData )
{
    // this is used to override the input pin with our own
    
    m_pInput = (CTransInPlaceInputPin*) new CGSEncoderInPin( this, phr );
    if( !m_pInput )
    {
        *phr = E_OUTOFMEMORY;
    }
}

STDMETHODIMP CGSEncoder::NonDelegatingQueryInterface( REFIID riid, void ** ppv) 
{
    if(riid == IID_IGSEncoder)
    {                
        return GetInterface((IGSEncoder *) this, ppv);
    }
    else
    {
        return CTransInPlaceFilter::NonDelegatingQueryInterface(riid, ppv);
    }
}



HRESULT CGSEncoder::CheckInputType( const CMediaType * pmt )
{
    CAutoLock lock( &m_Lock );

    if( *pmt->Type() == MEDIATYPE_Video )
    {
        if( *pmt->Subtype() == MEDIASUBTYPE_YUY2 )
        {
            return NO_ERROR;
        }
    }    
    return( E_UNEXPECTED );
}


//----------------------------------------------------------------------------
// this bit is almost straight out of the base classes.
// We override this so we can handle Transform( )'s error
// result differently.
//----------------------------------------------------------------------------

HRESULT CGSEncoder::Receive( IMediaSample * pms )
{
    // have the derived class transform the data
    HRESULT hr = Transform( pms );

    if( FAILED(hr) ) 
    {
        DbgLog((LOG_TRACE, 1, TEXT("Error from TransInPlace")));
    }
    return hr;
}


//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
HRESULT CGSEncoder::Transform( IMediaSample * pSample )
{
    static char pMessage[256];
    //Perform a Simple XOR to Test
    CAutoLock lock( &m_Lock );
    
    BYTE *pColorBuffer;

    pSample->GetPointer( &pColorBuffer );
    if( pColorBuffer )
    {
        m_Controller.ProcessImage( pColorBuffer );
        return m_pOutput->Deliver( pSample );
    }else
    {
        return E_UNEXPECTED;
    }
}


//----------------------------------------------------------------------------
//  Getting Kicked Off, so get ready
//----------------------------------------------------------------------------


HRESULT CGSEncoder::StartStreaming()
{
    CAutoLock lock( &m_Lock );
    
    LPBITMAPINFOHEADER pbiIn;

    if( IsEqualGUID(*m_pInput->CurrentMediaType().FormatType(), FORMAT_VideoInfo) ) 
    {
        pbiIn = &(((VIDEOINFOHEADER*)m_pInput->CurrentMediaType().Format())->bmiHeader);
    }

    if( pbiIn )
    {
        if( !m_Controller.Initialize( (short)pbiIn->biWidth, (short)pbiIn->biHeight, "D:\\MyRecordings\\MPEG2TST.MPG" ) )
        {
            return CTransInPlaceFilter::StartStreaming();
        }
    }
    return E_UNEXPECTED;
}


HRESULT CGSEncoder::StopStreaming()
{
    m_Controller.Finalize();
    return S_OK;
}
